"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const http_response_1 = __importDefault(require("./http.response"));
const http_error_1 = __importDefault(require("./http.error"));
class HttpClient {
    constructor(baseURL, timeout = 5000, retries = 3) {
        this.baseURL = baseURL;
        this.timeout = timeout;
        this.retries = retries;
        this.authToken = null;
        this.serviceSecret = null;
        this.requestInterceptors = [];
        this.responseInterceptors = [];
        this.loggingEnabled = false;
        this.responseLoggingEnabled = false;
    }
    setAuthToken(token) {
        this.authToken = token;
    }
    setServiceSecret(secret) {
        this.serviceSecret = secret;
    }
    enableLogging() {
        this.loggingEnabled = true;
    }
    enableResponseLogging() {
        this.responseLoggingEnabled = true;
    }
    addRequestInterceptor(interceptor) {
        this.requestInterceptors.push(interceptor);
    }
    addResponseInterceptor(interceptor) {
        this.responseInterceptors.push(interceptor);
    }
    applyRequestInterceptors(config) {
        return this.requestInterceptors.reduce((acc, interceptor) => interceptor(acc), config);
    }
    applyResponseInterceptors(response) {
        if (this.responseLoggingEnabled) {
            console.log(`Response: ${response.status} ${response.url}`, response);
        }
        return this.responseInterceptors.reduce((acc, interceptor) => interceptor(acc), response);
    }
    logRequest(method, url, options) {
        if (this.loggingEnabled) {
            console.log(`Request: ${method} ${url}`, options);
        }
    }
    handleResponse(response) {
        return __awaiter(this, void 0, void 0, function* () {
            let data = null;
            // Check if the response status indicates no content
            if (response.status !== 204 && response.status !== 205) {
                try {
                    // Attempt to parse the response body as JSON
                    data = yield response.json();
                }
                catch (error) {
                    console.warn('Failed to parse response JSON:', error);
                }
            }
            return new http_response_1.default(data, response.status, response.headers);
        });
    }
    handleError(response) {
        throw new http_error_1.default(response.status, response.statusText, response.url);
    }
    request(method_1, endpoint_1) {
        return __awaiter(this, arguments, void 0, function* (method, endpoint, data = null, config = {}) {
            const url = `${this.baseURL}${endpoint}`;
            const headers = Object.assign({ 'Content-Type': 'application/json' }, config.headers);
            if (this.authToken) {
                headers['Authorization'] = `Bearer ${this.authToken}`;
            }
            let options = {
                method,
                headers,
                body: data ? JSON.stringify(data) : null,
                credentials: config.credentials || 'same-origin',
            };
            options = this.applyRequestInterceptors(options);
            this.logRequest(method, url, options);
            for (let attempt = 0; attempt < this.retries; attempt++) {
                const controller = new AbortController();
                const timeoutId = setTimeout(() => controller.abort(), this.timeout);
                options.signal = controller.signal;
                try {
                    let response = yield fetch(url, options);
                    clearTimeout(timeoutId);
                    response = this.applyResponseInterceptors(response);
                    if (!response.ok) {
                        this.handleError(response);
                    }
                    return yield this.handleResponse(response);
                }
                catch (error) {
                    clearTimeout(timeoutId);
                    if (this.responseLoggingEnabled) {
                        console.error(error);
                    }
                    if (attempt < this.retries - 1) {
                        console.warn(`Retrying request to ${url} (attempt ${attempt + 1})`);
                    }
                    else {
                        console.error(`${method} request to ${url} failed after ${this.retries} attempts:`, error);
                        throw error;
                    }
                }
            }
        });
    }
    upload(endpoint_1, file_1) {
        return __awaiter(this, arguments, void 0, function* (endpoint, file, config = {}) {
            const url = `${this.baseURL}${endpoint}`;
            const formData = new FormData();
            formData.append('file', file);
            const headers = Object.assign({}, config.headers);
            if (this.authToken) {
                headers['Authorization'] = `Bearer ${this.authToken}`;
            }
            let options = {
                method: 'POST',
                headers,
                body: formData,
                credentials: config.credentials || 'same-origin',
            };
            options = this.applyRequestInterceptors(options);
            this.logRequest('POST', url, options);
            const controller = new AbortController();
            const timeoutId = setTimeout(() => controller.abort(), this.timeout);
            options.signal = controller.signal;
            try {
                let response = yield fetch(url, options);
                clearTimeout(timeoutId);
                response = this.applyResponseInterceptors(response);
                if (!response.ok) {
                    this.handleError(response);
                }
                return yield this.handleResponse(response);
            }
            catch (error) {
                clearTimeout(timeoutId);
                console.error(`File upload to ${url} failed:`, error);
                throw error;
            }
        });
    }
    download(endpoint_1) {
        return __awaiter(this, arguments, void 0, function* (endpoint, config = {}) {
            const url = `${this.baseURL}${endpoint}`;
            const headers = Object.assign({}, config.headers);
            if (this.authToken) {
                headers['Authorization'] = `Bearer ${this.authToken}`;
            }
            let options = {
                method: 'GET',
                headers,
                credentials: config.credentials || 'same-origin',
            };
            options = this.applyRequestInterceptors(options);
            this.logRequest('GET', url, options);
            const controller = new AbortController();
            const timeoutId = setTimeout(() => controller.abort(), this.timeout);
            options.signal = controller.signal;
            try {
                let response = yield fetch(url, options);
                clearTimeout(timeoutId);
                response = this.applyResponseInterceptors(response);
                if (!response.ok) {
                    this.handleError(response);
                }
                return yield response.blob();
            }
            catch (error) {
                clearTimeout(timeoutId);
                console.error(`File download from ${url} failed:`, error);
                throw error;
            }
        });
    }
    get(endpoint, config = {}) {
        return this.request('GET', endpoint, null, config);
    }
    post(endpoint, data, config = {}) {
        return this.request('POST', endpoint, data, config);
    }
    put(endpoint, data, config = {}) {
        return this.request('PUT', endpoint, data, config);
    }
    delete(endpoint, config = {}) {
        return this.request('DELETE', endpoint, null, config);
    }
}
exports.default = HttpClient;
